#ifndef SCENEOBJECTS_HPP_INCLUDED
#define SCENEOBJECTS_HPP_INCLUDED

#include "../../Core/Classes/GameModel.hpp"
#include "../../Core/Classes/DecorativeModel.hpp"
#include "../../Core/Classes/GroundModel.hpp"
#include "../../Core/Classes/WallModel.hpp"
#include "../../Core/Types/Tile.hpp"
#include "../../Core/Types/Convex.hpp"
#include <functional>
#include <cstdint>
#include <vector>
#include <string>

/** @addtogroup Models
* @{ */
class SceneObjects
{
    public:


        typedef enum MODEL_TYPE
        {
            GAME_MODEL = (1 << 1),
            DECORATIVE_MODEL = (1 << 2),
            GROUND_MODEL =  (1 << 3),
            WALL_MODEL =  (1 << 4),
            ALL = (GAME_MODEL | DECORATIVE_MODEL | GROUND_MODEL | WALL_MODEL)
        } MODEL_TYPE;

        static std::vector<Object> GetAll(MODEL_TYPE ModelTypes = ALL);
        static std::vector<Object> GetAll(const Tile& T, MODEL_TYPE ModelTypes = ALL);
        static std::vector<Object> GetAll(std::int32_t ID, MODEL_TYPE ModelTypes = ALL);
        static std::vector<Object> GetAll(const std::string& Name, MODEL_TYPE ModelTypes = ALL);
        static std::vector<Object> GetAll(const std::vector<std::int32_t>& PossibleIDs, MODEL_TYPE ModelTypes = ALL);
        static std::vector<Object> GetAll(const std::vector<std::string>& PossibleNames, MODEL_TYPE ModelTypes = ALL);

        static std::vector<GameModel> GetAll(const std::function<bool (GameModel&)>& Filter);
        static std::vector<DecorativeModel> GetAll(const std::function<bool (DecorativeModel&)>& Filter);
        static std::vector<GroundModel> GetAll(const std::function<bool (GroundModel&)>& Filter);
        static std::vector<WallModel> GetAll(const std::function<bool (WallModel&)>& Filter);

        static Object Get(const Tile& T, MODEL_TYPE ModelTypes = ALL);
        static Object Get(std::int32_t ID, MODEL_TYPE ModelTypes = ALL);
        static Object Get(const std::string& Name, MODEL_TYPE ModelTypes = ALL);
        static Object Get(const std::vector<std::int32_t>& PossibleIDs, MODEL_TYPE ModelTypes = ALL);
        static Object Get(const std::vector<std::string>& PossibleNames, MODEL_TYPE ModelTypes = ALL);

        static GameModel Get(const std::function<bool (GameModel&)>& Filter);
        static DecorativeModel Get(const std::function<bool (DecorativeModel&)>& Filter);
        static GroundModel Get(const std::function<bool (GroundModel&)>& Filter);
        static WallModel Get(const std::function<bool (WallModel&)>& Filter);

        static Tile GetTileOf(const GameModel& G);
        static Tile GetTileOf(const DecorativeModel& D);
        static Tile GetTileOf(const GroundModel& G);
        static Tile GetTileOf(const WallModel& W);

        static Convex GetConvexOf(const GameModel& G);
        static Convex GetConvexOf(const DecorativeModel& D);
        static Convex GetConvexOf(const GroundModel& G);
        static Convex GetConvexOf(const WallModel& W);
};

/** @} */

#endif // SCENEOBJECTS_HPP_INCLUDED
